
# q-expansion of newform 169.4.c.c, downloaded from the LMFDB on 17 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 169
weight = 4
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[2, [0, -1]]]
aps_data = [[3, -3], [1, -1], [9, 0], [0, 15], [-48, 48], [0, 0], [0, -45], [0, 6], [162, -162], [144, -144], [-264, 0], [303, -303], [-192, 192], [0, -97], [-111, 0], [-414, 0], [0, 522], [0, -376], [-36, 36], [0, 357], [1098, 0], [-830, 0], [438, 0], [-438, 438], [0, -852], [396, -396], [-182, 0], [612, -612], [-1083, 0], [0, -90], [2086, -2086], [-1467, 0], [0, -414], [0, 2419], [0, -930], [1683, 0], [1874, 0], [0, -1194], [-2388, 2388], [0, -1566], [-657, 657], [1222, 0], [0, -1260], [342, -342], [81, -81], [0, 1996], [-2833, 2833], [-3507, 3507], [0, -228], [-5493, 0], [-3627, 0], [-6075, 0], [0, 210], [0, 7092], [-5805, 5805], [-792, 792], [0, -5472], [2331, -2331], [0, -1384], [4062, 0], [-3764, 3764], [0, 4227], [-306, 0], [2106, 0], [10051, 0], [2154, 0], [0, 10770], [2171, 0], [0, 7047], [-6873, 6873], [-9318, 9318], [-4128, 0], [2536, -2536], [0, 92], [10182, -10182], [0, -579], [-2106, 0], [0, -1974], [11886, -11886], [0, 1254], [-5823, 5823], [7341, 0], [-7485, 7485], [0, -15203], [-1762, 1762], [-7317, 0], [0, -5016], [9870, -9870], [0, -14541], [2112, 0], [-3276, 0], [-15453, 15453], [0, -3660], [747, -747], [15804, 0], [0, 12078], [16110, -16110], [3915, 0], [-16184, 16184], [7923, 0], [-14389, 0], [-10383, 10383], [0, -16425], [12213, -12213], [6383, 0], [-6426, 0], [-21330, 21330], [-12084, 0], [2394, 0], [21971, -21971], [0, 15406], [-9630, 9630], [0, 14748], [3672, 0], [0, -19875], [0, 1710], [0, -16452], [25902, -25902], [-18108, 18108], [0, 32904], [15318, -15318], [-7729, 7729], [19242, 0], [0, 22518], [9168, -9168], [-1170, 0], [0, -1662], [0, 30960], [8372, 0], [2739, 0], [-6756, 6756], [29643, -29643], [18128, -18128], [6410, -6410], [0, 28290], [-27960, 27960], [0, -5649], [0, 756], [0, 31194], [-17055, 17055], [-35520, 0], [1095, -1095], [0, -2554], [-21522, 0], [-13124, 13124], [0, -23424], [-31077, 0], [19422, 0], [1744, 0], [-19179, 0], [0, 29217], [-15633, 15633], [-30589, 0], [25884, -25884], [-12305, 12305], [29772, 0], [0, -47644], [0, 21924], [32398, 0], [-2097, 0], [-20016, 20016], [0, 24993], [40959, 0], [0, 48933], [47388, -47388], [-16803, 0], [57526, -57526], [0, 25000]]
